Class: Relaton::Db
- Inherits:
-
Object
- Object
- Relaton::Db
- Defined in:
- lib/relaton/db.rb
Class Method Summary collapse
- .flush_caches(gcache, lcache) ⇒ Object
- .global_bibliocache_name ⇒ Object
-
.init_bib_caches(**opts) ⇒ Relaton::Db
Initialse and return relaton instance, with local and global cache names.
- .local_bibliocache_name(cachename) ⇒ Object
Instance Method Summary collapse
-
#clear ⇒ Object
Clear global and local databases.
-
#docid_type(code) ⇒ Array
The document identifier class corresponding to the given code.
-
#fetch(text, year = nil, opts = {}) ⇒ nil, ...
The class of reference requested is determined by the prefix of the reference: GB Standard for gbbib, IETF for ietfbib, ISO for isobib, IEC or IEV for iecbib,.
-
#fetch_all(text = nil, edition: nil, year: nil) ⇒ Array
fetch all standards from DB.
-
#fetch_async(ref, year = nil, opts = {}, &block) ⇒ RelatonBib::BibliographicItem, ...
Fetch asynchronously.
- #fetch_db(code, year = nil, opts = {}) ⇒ Object
- #fetch_std(code, year = nil, stdclass = nil, opts = {}) ⇒ nil, ...
-
#initialize(global_cache, local_cache) ⇒ Db
constructor
A new instance of Db.
- #load_entry(key) ⇒ Hash
-
#mv(new_dir, type: :global) ⇒ String?
Move global or local caches to anothe dirs.
- #save_entry(key, value) ⇒ Object
-
#to_xml ⇒ String
list all entries as a serialization.
Constructor Details
#initialize(global_cache, local_cache) ⇒ Db
Returns a new instance of Db.
5 6 7 8 9 10 11 12 13 |
# File 'lib/relaton/db.rb', line 5 def initialize(global_cache, local_cache) @registry = Relaton::Registry.instance gpath = global_cache && File.(global_cache) @db = open_cache_biblio(gpath) lpath = local_cache && File.(local_cache) @local_db = open_cache_biblio(lpath) @queues = {} @semaphore = Mutex.new end |
Class Method Details
.flush_caches(gcache, lcache) ⇒ Object
620 621 622 623 |
# File 'lib/relaton/db.rb', line 620 def flush_caches(gcache, lcache) FileUtils.rm_rf gcache unless gcache.nil? FileUtils.rm_rf lcache unless lcache.nil? end |
.global_bibliocache_name ⇒ Object
625 626 627 |
# File 'lib/relaton/db.rb', line 625 def global_bibliocache_name "#{Dir.home}/.relaton/cache" end |
.init_bib_caches(**opts) ⇒ Relaton::Db
Initialse and return relaton instance, with local and global cache names
613 614 615 616 617 618 |
# File 'lib/relaton/db.rb', line 613 def init_bib_caches(**opts) # rubocop:disable Metrics/CyclomaticComplexity globalname = global_bibliocache_name if opts[:global_cache] localname = local_bibliocache_name(opts[:local_cache]) flush_caches globalname, localname if opts[:flush_caches] Relaton::Db.new(globalname, localname) end |
.local_bibliocache_name(cachename) ⇒ Object
629 630 631 632 633 634 |
# File 'lib/relaton/db.rb', line 629 def local_bibliocache_name(cachename) return nil if cachename.nil? cachename = "relaton" if cachename.empty? "#{cachename}/cache" end |
Instance Method Details
#clear ⇒ Object
Clear global and local databases
27 28 29 30 31 32 33 34 |
# File 'lib/relaton/db.rb', line 27 def clear @db&.clear @local_db&.clear @registry.processors.each_value do |p| require p.short.to_s p.remove_index_file if p.respond_to? :remove_index_file end end |
#docid_type(code) ⇒ Array
The document identifier class corresponding to the given code
163 164 165 166 167 |
# File 'lib/relaton/db.rb', line 163 def docid_type(code) stdclass = @registry.class_by_ref(code) or return [nil, code] _, code = strip_id_wrapper(code, stdclass) [@registry[stdclass].idtype, code] end |
#fetch(text, year = nil, opts = {}) ⇒ nil, ...
The class of reference requested is determined by the prefix of the reference: GB Standard for gbbib, IETF for ietfbib, ISO for isobib, IEC or IEV for
iecbib,
65 66 67 68 69 70 71 72 73 74 75 76 |
# File 'lib/relaton/db.rb', line 65 def fetch(text, year = nil, opts = {}) reference = text.strip stdclass = @registry.class_by_ref(reference) || return processor = @registry[stdclass] ref = if processor.respond_to?(:urn_to_code) processor.urn_to_code(reference)&.first else reference end ref ||= reference result = combine_doc ref, year, opts, stdclass result || check_bibliocache(ref, year, opts, stdclass) end |
#fetch_all(text = nil, edition: nil, year: nil) ⇒ Array
fetch all standards from DB
89 90 91 92 93 94 95 96 97 98 |
# File 'lib/relaton/db.rb', line 89 def fetch_all(text = nil, edition: nil, year: nil) result = [] db = @db || @local_db if db result += db.all do |file, xml| search_xml file, xml, text, edition, year end.compact end result end |
#fetch_async(ref, year = nil, opts = {}, &block) ⇒ RelatonBib::BibliographicItem, ...
Fetch asynchronously
110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 |
# File 'lib/relaton/db.rb', line 110 def fetch_async(ref, year = nil, opts = {}, &block) # rubocop:disable Metrics/AbcSize,Metrics/MethodLength stdclass = @registry.class_by_ref ref if stdclass unless @queues[stdclass] processor = @registry[stdclass] threads = ENV["RELATON_FETCH_PARALLEL"]&.to_i || processor.threads wp = WorkersPool.new(threads) do |args| args[3].call fetch(*args[0..2]) rescue RelatonBib::RequestError => e args[3].call e rescue StandardError => e Util.error "`#{args[0]}` -- #{e.message}" args[3].call nil end @queues[stdclass] = { queue: SizedQueue.new(threads * 2), workers_pool: wp } Thread.new { process_queue @queues[stdclass] } end @queues[stdclass][:queue] << [ref, year, opts, block] else yield nil end end |
#fetch_db(code, year = nil, opts = {}) ⇒ Object
79 80 81 82 |
# File 'lib/relaton/db.rb', line 79 def fetch_db(code, year = nil, opts = {}) opts[:fetch_db] = true fetch code, year, opts end |
#fetch_std(code, year = nil, stdclass = nil, opts = {}) ⇒ nil, ...
150 151 152 153 154 155 156 157 158 |
# File 'lib/relaton/db.rb', line 150 def fetch_std(code, year = nil, stdclass = nil, opts = {}) std = nil @registry.processors.each do |name, processor| std = name if processor.prefix == stdclass end std = @registry.class_by_ref(code) or return nil unless std check_bibliocache(code, year, opts, std) end |
#load_entry(key) ⇒ Hash
171 172 173 |
# File 'lib/relaton/db.rb', line 171 def load_entry(key) (@local_db && @local_db[key]) || @db[key] end |
#mv(new_dir, type: :global) ⇒ String?
Move global or local caches to anothe dirs
19 20 21 22 23 24 |
# File 'lib/relaton/db.rb', line 19 def mv(new_dir, type: :global) case type when :global then @db&.mv new_dir when :local then @local_db&.mv new_dir end end |
#save_entry(key, value) ⇒ Object
178 179 180 181 |
# File 'lib/relaton/db.rb', line 178 def save_entry(key, value) @db.nil? || (@db[key] = value) @local_db.nil? || (@local_db[key] = value) end |
#to_xml ⇒ String
list all entries as a serialization
185 186 187 188 189 190 |
# File 'lib/relaton/db.rb', line 185 def to_xml db = @local_db || @db || return Nokogiri::XML::Builder.new(encoding: "UTF-8") do |xml| xml.documents { xml.parent.add_child db.all.join(" ") } end.to_xml end |